<?php
namespace Vtbpay\Classes;

class VtbApi extends Api
{
    private static $tokenUrl;

    private static $orderUrl;

    private static $mode;

    private static $clientMerchant;
    
    private static $clientData = [
        'grant_type' => 'client_credentials'
    ];

    private static $modeUrls = [
        'PROD' => [
            'TOKEN' => 'https://open.api.vtb.ru:443/passport/oauth2/token',
            'ORDER' => 'https://gw.api.vtb.ru/openapi/smb/efcp/e-commerce/v1/orders'
        ],
        'TEST' => [
            'TOKEN' => 'https://auth.bankingapi.ru/auth/realms/kubernetes/protocol/openid-connect/token',
            'ORDER' => 'https://hackaton.bankingapi.ru/api/smb/efcp/e-commerce/api/v1/orders'
        ]
    ];

    public function __construct(
        $clientId = false,
        $clientSecret = false,
        $test = false,
        $merchant = ''
    ) {
        self::$clientData['client_id'] = trim($clientId);
        self::$clientData['client_secret'] = trim($clientSecret);
        self::$clientMerchant = trim($merchant);
        self::$mode = $test ? 'TEST': 'PROD';

        self::$tokenUrl = self::$modeUrls[self::$mode]['TOKEN'];
        self::$orderUrl = self::$modeUrls[self::$mode]['ORDER'];
    }
    

    public function getAccessToken() {
        return $this->sendRequest(
            self::$tokenUrl,
            http_build_query(self::$clientData),
            [
                'Content-Type: application/x-www-form-urlencoded',
                'Merchant-Authorization: ' . self::$clientMerchant
            ]
        );
    }
    

    public function getOrderLink(
        $orderId,
        $email,
        $orderCreate,
        $amount,
        $returnUrl
    ) {
        $result = false;
        $headers = $this->getAutorizationHeaders();
        $merchant = self::$clientMerchant;

        if ($headers) {
            $headers[] = 'Content-Type: application/json';
            if ($merchant) {
                $headers[] = 'Merchant-Authorization: ' . $merchant;
            }

            $timestamp = $orderCreate + 86400; // +1d
            $expire = date('Y-m-d\TH:i:s.v\Z', $timestamp); // ISO 8601

            $args = [
                'orderId' => $orderId,
                'orderName' => 'Заказ #' . $orderId,
                'customer' => [
                    'email' => $email
                ],
                'amount' => [
                    'value' => $amount,
                    'code' => 'RUB'
                ],
                'expire' => $expire,
                'returnUrl' => $returnUrl
            ];

            $result = $this->sendRequest(
                self::$orderUrl,
                $args,
                $headers
            );
        }

        return $result;
    }
    

    public function getOrderInfo(string $orderId)
    {
        $result = false;
        $headers = $this->getAutorizationHeaders();
        $merchant = self::$clientMerchant;

        if ($headers) {
            if ($merchant) {
                $headers[] = 'Merchant-Authorization: ' . $merchant;
            }

            $result = $this->sendRequest(
                self::$orderUrl . '/' . $orderId,
                [],
                $headers,
                'GET'
            );
        }

        return $result;
    }
    

    private function getAutorizationHeaders()
    {
        $token = $this->getAccessToken();
        
        if(!isset($token['access_token'])) {
          
            $errorDescription = '';
            if (
                isset($token['error_description']) && 
                !empty($token['error_description'])
            ) $errorDescription .= ' Error description: ' . $token['error_description'];
            
            throw new \Exception('Error getting access token.' . $errorDescription);
        }

        return [
            'X-IBM-Client-Id: ' . explode('@', self::$clientData['client_id'])[0],
            'Authorization: ' . $token['token_type'] . ' ' . $token['access_token'],
        ];
    }
}
